"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.contextRules = exports.isContext = exports.isInContext = exports.currentFunction = exports.createRule = void 0;
const utils_1 = require("@typescript-eslint/utils");
const ast_spec_1 = require("@typescript-eslint/types/dist/generated/ast-spec");
exports.createRule = utils_1.ESLintUtils.RuleCreator((name) => `https://example.com/rule/${name}`);
const publicRoleSplitter = "_";
const privateRoleSplitter = "__";
const isContextRegexp = /\W*@DCI-context\b/i;
class Context {
    constructor(context, func) {
        this.roles = new Map();
        this.funcMap = new Map();
        this.func = func;
        if (func.body.type != ast_spec_1.AST_NODE_TYPES.BlockStatement) {
            context.report({
                loc: func.body.loc,
                message: "A Context must be defined with a block statement (curly braces)",
            });
            return;
        }
        const potentialRoles = new Map(func.params.filter((p) => p.type == ast_spec_1.AST_NODE_TYPES.Identifier).map((i) => [i.name, i]));
        const statements = func.body.body;
        // Check if a top-level statement exists as a potential Role.
        for (const s of statements) {
            if (s.type == ast_spec_1.AST_NODE_TYPES.VariableDeclaration &&
                s.declarations.length == 1 &&
                s.declarations[0]?.type == ast_spec_1.AST_NODE_TYPES.VariableDeclarator &&
                s.declarations[0]?.id.type == ast_spec_1.AST_NODE_TYPES.Identifier) {
                const declaration = s.declarations[0].id;
                potentialRoles.set(declaration.name, declaration);
            }
        }
        //console.log(potentialRoles.keys());
        const functions = statements.filter((n) => n.type == ast_spec_1.AST_NODE_TYPES.FunctionDeclaration);
        // Check if a function is a RoleMethod
        for (const func of functions) {
            const rm = this.roleMethodCall(func.id?.name);
            if (!rm)
                continue;
            const contextRm = {
                role: rm.role,
                method: rm.method,
                isPrivate: rm.isPrivate,
                func,
            };
            if (!this.roles.has(rm.role)) {
                // Check if a potential role exist, if so, add the Role.
                if (potentialRoles.has(rm.role)) {
                    this.roles.set(rm.role, {
                        id: potentialRoles.get(rm.role),
                        methods: [contextRm],
                    });
                }
                else {
                    context.report({
                        node: func,
                        message: `Role "${rm.role}" not found for RoleMethod "${rm.method}"`,
                    });
                }
            }
            else {
                this.roles.get(rm.role)?.methods?.push(contextRm);
                this.funcMap.set(func, contextRm);
            }
        }
    }
    roleMethodFromFunc(func) {
        return this.funcMap.get(func);
    }
    isRoleMethod(name) {
        return name && name.includes(publicRoleSplitter);
    }
    roleMethodCall(name) {
        if (!name || !this.isRoleMethod(name))
            return null;
        const isPrivate = name.includes(privateRoleSplitter);
        const parts = name.split(isPrivate ? privateRoleSplitter : publicRoleSplitter);
        return { role: parts[0], method: parts[1], isPrivate };
    }
}
const _currentContext = [];
let _currentFunction = null;
const currentContext = () => _currentContext[_currentContext.length - 1];
const currentFunction = () => _currentFunction;
exports.currentFunction = currentFunction;
const isInContext = () => currentContext();
exports.isInContext = isInContext;
const isContext = (func) => func && currentContext()?.func === func ? currentContext() : null;
exports.isContext = isContext;
const contextRules = (context, rule) => {
    rule[" FunctionDeclaration:exit"] = (func) => {
        _currentFunction = null;
        if (currentContext()?.func === func) {
            //console.log("Exiting Context " + func?.id?.name);
            _currentContext.pop();
        }
    };
    rule[" FunctionDeclaration"] = (func) => {
        _currentFunction = func;
        const currentCtx = currentContext();
        // Skip if we're already in the same context
        if (currentCtx?.func === func)
            return;
        const source = context.getSourceCode();
        const comments = currentCtx
            ? source.getCommentsBefore(func)
            : [func, ...context.getAncestors()].flatMap((n) => source.getCommentsBefore(n));
        const commentStr = comments.map((c) => c.value).join(" ");
        const hasContextComment = commentStr.match(isContextRegexp);
        if (hasContextComment) {
            //console.log("Entering Context " + func.id?.name);
            _currentContext.push(new Context(context, func));
        }
    };
    return rule;
};
exports.contextRules = contextRules;
